@@ -20,6 +20,7 @@
 
 int command_done;
 int VERBOSE;
+int VERBOSE_EXIT;
 int STRICT;
 int INIT_MODE;
 int FUZZING;
@@ -30,3 +31,5 @@ char* PATH;
 struct Token* token;
 /* Env linked-list; stores the environment variables */
 struct Token* env;
+/* Alias linked-list; stores the aliases */
+struct Token* alias;
